/**
 * DevExtreme (core/nativeRendererStrategy.js)
 * Version: 17.1.2-pre-beta
 * Build date: Thu May 04 2017
 *
 * Copyright (c) 2012 - 2017 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
module.exports = {
    createElement: function(tagName, text) {
        if ("#text" === tagName) {
            return document.createTextNode(text)
        }
        return document.createElement("string" === typeof tagName ? tagName : "div")
    },
    removeElement: function(element) {
        var parentNode = element && element.parentNode;
        if (parentNode) {
            parentNode.removeChild(element)
        }
    },
    insertElement: function(parentElement, newElement, nextSiblingElement) {
        if (parentElement && newElement && parentElement !== newElement) {
            parentElement.insertBefore(newElement, nextSiblingElement)
        }
    },
    setAttribute: function(element, name, value) {
        if (void 0 !== value && null !== value) {
            element.setAttribute(name, value)
        } else {
            element.removeAttribute(name)
        }
    },
    setProperty: function(element, name, value) {
        element[name] = value
    },
    setEvent: function(element, name, value) {
        element.addEventListener(name, value)
    },
    setText: function(element, text) {
        if (element) {
            element.textContent = text
        }
    },
    setClass: function(element, className, isAdd) {
        if (element.nodeType) {
            if (element.classList) {
                if (isAdd) {
                    element.classList.add(className)
                } else {
                    element.classList.remove(className)
                }
            } else {
                var classNames = element.className.split(" ");
                var classIndex = classNames.indexOf(className);
                if (isAdd && classIndex < 0) {
                    element.className = element.className ? element.className + " " + className : className
                }
                if (!isAdd && classIndex >= 0) {
                    classNames.splice(classIndex, 1);
                    element.className = classNames.join(" ")
                }
            }
        }
    },
    setStyle: function(element, name, value) {
        element.style[name] = value || ""
    }
};
